#!/bin/sh
#
# bb-page1.sh
# BIG BROTHER PAGER SCRIPT
# Sean MacGuire - The MacLawran Group Inc.
# Version 1.09
# Feb 2nd 1999
#
# This program is Copyright (c) 1997-1999
# The MacLawran Group Inc.
# All Rights Reserved
#
# Format: bb-page1.sh <NUMERIC-MESSAGE> <ALPHA-MESSAGE>
#
# Modified to be generic front-end for all paging systems
# and gets all info out of etc/bbdef.sh file
#

#
# Changed 970111 by Jacob Lundqvist jaclu@ibk.se
#
#  Included working SMS and improved NUMERIC paging
#  
#  Now kermit attempts are queued!
#

if test "$#" = "1"	# CALLED AS A SINGLE ARG FROM bbd
then
	set $1		# PUT INTO THE ENVIRONMENT
fi

if test "$#" -lt "2"	# IF THERE AREN'T AT LEAST TWO ARGUMENTS
then
	echo "bb-page1.sh:	Incorrect number of arguments"
	echo "Format:		bb-page1.sh <NUMERIC-MESSAGE> <ALPHA-MESSAGE>"
	exit 1
fi

BBNUMERIC=`echo $1`	# THE NUMERIC MESSAGE
shift;
BBALPHA="$*"		# THE ALPHA MESSAGE
BBHOSTSVC=$1
BBCOLORLEVEL=$2

# Build the real numeric, BBHOSTIP is required for HOST notification
OLDIFS=$IFS
IFS='.'
set $BBNUMERIC
BBSVCNUM=$1
BBHOSTIP=$2
BBNUMERIC=${BBSVCNUM}${BBHOSTIP}
IFS=$OLDIFS

if test ! "$BBTMP" 			# GET DEFINITIONS IF NEEDED
then
	. $BBHOME/etc/bbdef.sh          # INCLUDE STANDARD DEFINITIONS
fi

#
# GET THE NUMERIC PREFIX/SUFFIX TO DIAL
#

PREFIX=""
prefixline=`grep "^prefix:" $BBHOME/etc/bbwarnsetup.cfg` 2>/dev/null
if [ "$?" -eq 0 ]
then
	set $prefixline >/dev/null 2>&1
	if [ "$#" -gt 1 ]
	then
		shift           # remove prefix token from line
		PREFIX="$*"
	fi      
fi

SUFFIX=""
suffixline=`grep "^suffix:" $BBHOME/etc/bbwarnsetup.cfg` 2>/dev/null
if [ "$?" -eq 0 ]
then
	set $suffixline >/dev/null 2>&1
	if [ "$#" -gt 1 ]
	then
		shift           # remove suffix token from line
		SUFFIX="$*"
	fi      
fi

#
# GET THE PAGEDELAY VALUE
#

PAGEDELAY=15
pagedelayline=`grep "^pagedelay:" $BBHOME/etc/bbwarnsetup.cfg` 2>/dev/null
if [ "$?" -eq 0 ]
then
	set $pagedelayline >/dev/null 2>&1
	if [ "$#" -gt 1 ]
	then
		shift           # remove pagedelay token from line
		param="$1"
		# Check if it's a valid parameter
		expr "$param" + 0 >/dev/null 2>&1
		if [ "$?" -eq 0 ]
		then
			PAGEDELAY="$param"
		fi
	fi      
fi

#
# GET PAGELEVELSMAIL
#

PAGELEVELSMAIL="nocolordefined"
pageslevelsmailline=`grep "^pagelevelsmail:" $BBHOME/etc/bbwarnsetup.cfg` 2>/dev/null
if [ "$?" -eq 0 ]
then
	set $pagelevelsmailline >/dev/null 2>&1
	if [ "$#" -gt 1 ]
	then
		shift           # remove pagelevelsmail token from line
		param="$*"

		# Check if it's a valid parameter, remove bad values

		for color in $param
		do
			case $color in

			yellow | red | purple )

				if [ "$PAGELEVELSMAIL" = "nocolordefined" ]
				then
					PAGELEVELSMAIL="$color"
				else
					PAGELEVELSMAIL="$PAGELEVELSMAIL | $color"
				fi
				;;

			* )
				;;

			esac
		done
	fi      
fi

#
# GET PAGETYPE
# THIS CODE IS ALSO IN bb-doack.sh
# ANY CHANGES HERE MUST BE DONE THERE

PAGETYPE="RCPT"
pagetypeline=`grep "^pagetype:" $BBHOME/etc/bbwarnsetup.cfg` 2>/dev/null
if [ "$?" -eq 0 ]
then
	set $pagetypeline >/dev/null 2>&1
	if [ "$#" -gt 1 ]
	then
		shift           # remove pagertype token from line
		param="$1"

		# Check if it's a valid parameter, 
		# Set it if it's valid

		case $param in

		RCPT | GROUP | HOST | EVENT )

			PAGETYPE=$param
			;;

		* )
			;;

		esac
	fi      
fi

# Create a file that contains all the group/hosts relationships
if [ "$PAGETYPE" = "GROUP" ]
then
	# Get the service name
	OLDIFS=$IFS
	IFS="."
	set $BBHOSTSVC
	shift `expr $# - 1`
	svc=$1
	IFS=$OLDIFS

	# Build a file with group/hosts tag
	$RM -f $BBTMP/BBGRP.$$
	gblgrp="global-grp"
	currgrp=$gblgrp
	# Read all entries of bb-hosts file
	$CAT $BBHOSTS | $GREP -v "^#" |
	while read line
	do
		# Is it a host definition ?
		hostsline=`echo "$line" | $GREP "^[0-9]" 2>/dev/null`
		if [ "$?" -eq 0 ]
		then
			set $hostsline
			if [ "$FQDN" != "TRUE" ]
			then
				OLDIFS=$IFS
				IFS="."
				set $2
				IFS=$OLDIFS
			else
				# Position at the host name
				shift
			fi
			echo "$currgrp $1.$svc" >> $BBTMP/BBGRP.$$
		else
			# Is it a group definition ?
			grpline=`echo "$line" | $GREP "^group" 2>/dev/null`
			if [ "$?" -eq 0 ]
			then
				set $grpline
				shift
				currgrp=`echo $* | $SED 's/[^a-zA-Z0-9]//g'`
				if [ "$currgrp" = "" ]
				then
					currgrp=$gblgrp
				fi
			fi
		fi
	done

	# At this point there must be at least one entry
	# in the $BBGRP.$$ file
	grpline=`$GREP $BBHOSTSVC $BBTMP/BBGRP.$$` 2>/dev/null
	if [ "$?" -eq 0 ]
	then
		set $grpline
		bbgroup=$1
	else
		# Just in case that nothing matches set in global group
		bbgroup=$gblgrp
	fi
	$RM -f $BBTMP/BBGRP.$$
fi

#
# NOW WE DECIDE HOW TO HANDLE THINGS
# PAGER CONTAINS A LIST OF PAGERS TO CALL, ARGS AS FOLLOWS
#
#	NUMBER 		SEND DUMB OLDNUMERIC MESSAGE
#	XXX@YYYY	SEND e-mail TO THIS ADDRESS
#	sms-NUMBER 	SEND sms MESSAGE
#	alpha-NUMBER	SEND alpha MESSAGE
#
# EXAMPLE:
#
# 	PAGER="123123 sean@MacLawran.ca sms-123123"
# 	THIS SHOULD SEND A NUMERIC PAGE TO 123123, 
#	MAIL TO sean@MacLawran.ca
# 	AND AN SMS-PAGE TO 123123
#
# SMM 1.07c: CHANGED THE if STATEMENTS TO A case STATEMENT
#	SHOULD BE MUCH EASIER TO MODIFY NOW.
#

rcptnum=10			# Start at 10, no need for leading zeros
for inpage in `echo $PAGER`		# FOR EACH PAGER ARGUMENT...
do

	#
	# SMM: NEW IN 1.05 - PAGE ONLY IF THE REQUIRED PAGEDELAY
	# HAS PASSED ALREADY...
	#
	# SMM: NEW IN 1.08a - AND ISN'T FROM THE BBDISPLAY PAGE BUTTON
	# Thanks to: "Eric J. Frisch" <efrisch@magi.com>
	#
	echo $BBNUMERIC | $GREP "^${PAGEHELPCODE}" > /dev/null
	if test $? != 0
	then
		$BBHOME/bin/touchtime 0  ${BBTMP}/timenow.$$

		# IS there an overiding PAGEDELAY value ?
		OLDIFS=$IFS
		IFS=:
		set $inpage
		page=$1
		IFS=$OLDIFS

		pagefilepfx="np_"
		# Notification tagfile always has the recipient
		pagefile="${pagefilepfx}${page}"
		if [ "$PAGETYPE" = "EVENT" ]
		then
			pagefile="${pagefile}_${BBNUMERIC}"
		elif [ "$PAGETYPE" = "GROUP" ]
		then
			pagefile="${pagefile}_${bbgroup}"
		elif [ "$PAGETYPE" = "HOST" ]
		then
			pagefile="${pagefile}_${BBHOSTIP}"
		fi
	
		$FIND $BBTMP -name "${pagefile}" -newer ${BBTMP}/timenow.$$ -print | $GREP "${pagefile}" > /dev/null
		retcode=$?
		$RM -f  ${BBTMP}/timenow.$$
		if test $retcode = 0
		then
			# echo "Page delay not expired... page cancelled..."
			continue
		fi

		#
		# IT'S OK TO PAGE, SO SET THE TIME WE CAN SEND THE NEXT ONE...
		#
		DELAY=$PAGEDELAY
		shift >/dev/null 2>&1
		if [ "$#" -gt 0 ]	# Is there another arg ?
		then
			TESTDELAY=`echo $1|$SED 's/[0-9]//g'`
			if [ -n "$TESTDELAY" ]	# Was delay properly formed ?
			then
				echo "bb-page1.sh: Delay in cfg file is invalid for $page"
				DELAY=$PAGEDELAY	# no !
			else
				DELAY=$1
			fi
		fi

		# Create Ack number
		# Process ID + recipient ID (10-99)
		# processes id are zero filled, this allows easy
		# checking for invalid ack numbers in bb-ack.html
		# by using $num == `expr $num + 0`
		# AIX PID's can be > 99999, so make sure < 100000
		pid=`expr $$ % 100000`
		if [ "$pid" -lt 10 ]
		then
			randval="${pid}0000${rcptnum}"
		elif [ "$pid" -lt 100 ]
		then
			randval="${pid}000${rcptnum}"
		elif [ "$pid" -lt 1000 ]
		then
			randval="${pid}00${rcptnum}"
		elif [ "$pid" -lt 10000 ]
		then
			randval="${pid}0${rcptnum}"
		else
			randval="${pid}${rcptnum}"
		fi	
		#randval=`$BBHOME/bin/random "${pagefile}"`
		echo "$randval $BBHOSTSVC $BBCOLORLEVEL" > ${BBTMP}/${pagefile}
		$BBHOME/bin/touchtime $DELAY ${BBTMP}/${pagefile}
		rcptnum=`expr $rcptnum + 1`
	else
		page="$inpage"
	fi

	if [ "$randval" != "" ]
	then
		randvaltag="[$randval] "
		randvalsubjtag=" - $randval"
	else
		randvaltag=""
		randvalsubjtag=""
	fi

	if [ -d "$BBHOME/www/html" ]
	then
		case $BBHOSTSVC in

		notify-admin* | unmatched-* )

			htmllink=""
			;;

		* )

			htmllink="

Please see: ${BBWEBHTMLLOGS}${BBHOSTSVC}.html"
			;;
		esac
	else
		htmllink=""
	fi

	case $BBCOLORLEVEL in

	$PAGELEVELSMAIL )

		# Send only e-mail for if the status message color matches one
		# color in PAGELEVELSMAIL (set in bbwarnsetup.cfg)

		case $page in

		*@* )		# echo "EMAIL"		# NOTIFY BY e-mail
				echo "${randvaltag}${BBALPHA}${htmllink}" | $MAIL "!BB${randvalsubjtag}! $BBHOSTSVC - $BBNUMERIC" "$page"
				;;
		* )
				# Color should only be sent by mail and this recipient isn't mail ...
				;;
		esac
		;;

	* )
	
		case $page in

		*@* )		# echo "EMAIL"		# NOTIFY BY e-mail
				echo "${randvaltag}${BBALPHA}${htmllink}" | $MAIL "!BB${randvalsubjtag}! $BBHOSTSVC - $BBNUMERIC" "$page"
				;;

		[0-9]* )	# echo "NUMERIC PAGING"
				$BBHOME/bin/sendmsg $BBHOME/etc/numeric.scr ${PREFIX}$page${SUFFIX} ${BBNUMERIC}${randval}
				;;

		sms-* )		# echo "SMS"
				set `echo $page | sed "s/-/ /"`
				NUMBER=$2
				#
				# Message must be stored in a file, 
				# for the sms kermit script
				#
				SMSFILE=${BBTMP}/sms-$$
				echo "${randvaltag}$BBALPHA" > $SMSFILE
				#
				# Note!!!
				# =======
				# If you use a dialing prefix, it must be manually set in
				# sms.scr
				# Here we only send the numbers to page, not the paging
				# server
				#
				$BBHOME/bin/sendmsg $BBHOME/etc/sms.scr $SMSFILE $NUMBER
				$RM -f $SMSFILE
				;;

		alpha-* )	# echo "ALPHA PAGING"
				#
				# SUPPORT FOR ALPHA PAGING OR WHATEVER...
				# THIS ISN'T IMPLEMENTED YET!
				# SUGGESTIONS ANYONE?
				#
				set `echo $page | sed "s/-/ /"`
				NUMBER=$2
				;;

		* )		# echo "DEFAULT"
				;;
		esac
		;;
	esac
done

exit $?
