/*
 * bb.c 
 *
 * BIG BROTHER CLIENT PROGRAM
 * Sean MacGuire - The MacLawran Group Inc.
 * Version 1.09
 * Feb 2nd, 1999
 *
 * This program is Copyright (c) 1997-1999
 * The MacLawran Group Inc.
 * All Rights Reserved
 *
 * SENDS THE INPUT LINE TO THE DAEMON ON THE BIG BROTHER PORT
 * Format: bb IP-ADDR [<LINE>|-]
 *
 *	IP-ADDR:	IP-ADDR OF THE SERVER
 *	LINE:		<page|status> <DATA>
 *		DATA:	for page:	<NUMERIC MESSAGE><status-msg>
 *			for status:	<machine><color-code><status-msg>
 *
 *	-		Will read from standard input a one line status
 *				message.
 */

#include "bb.h"		/* THE BIG BROTHER INCLUDE FILE */

char *ipaddr;

jmp_buf env;  

main(argc, argv)
int argc;
char *argv[];
{
char data[256];

	if (argc != 3) {
		fprintf(stderr, "bb: incorrect number of arguments\n");
		fprintf(stderr, "Format: <IP-ADDR> <DATA>\n");
		exit(1);
	}

	if( argv[2][0] == '-' ) {
	  debug("bb gets its input from stdin\n");
	  while (fgets(data,sizeof(data)-1,stdin) != NULL) {
	    debug("data came in [%s]\n",data);
	    bbnotify(argv[1],data);
	  }
	  debug("Done\n");
	}
	else {
	  bbnotify(argv[1],argv[2]);
	}

	exit(0);
}

bbnotify(ip,data)
char *ip;                               /* IP ADDRESS */
char *data;                             /* DATA TO SEND ACROSS */
{
  char *bbpage = NULL;
  char *pagelevels = NULL;
  char v1[16],v2[128],v3[16],v4[16];

  debug("Sending: ** %s ** to %s\n",data,ip);
  bbsend(ip, data);     /* SEND IT ACROSS */

  /* If BBPAGE is defined then send also a msg if it's a red alert */
  /* What if BBPAGE == BBDISP ? for now send two messages */
  /* keep it simple ... */
  bbpage = (char *) getenv("BBPAGE");
  pagelevels = (char *) getenv("PAGELEVELS");
  if( ! pagelevels || (strlen(pagelevels) == 0) )
	pagelevels = PAGELEVELSDEFAULT;
  if( (bbpage != NULL) && (pagelevels != NULL) ) {
    /* %*s could have been used for v2 but */
    /* Linux thought otherwise i.e. bug or I misunderstood the %*s (skip) */
    sscanf(data,"%15s %127s %16s %*s",v1,v2,v3,v4);
    debug("v1:[%s] v2:[%s] v3:[%s] v4:[%s]\n",v1,v2,v3,v4);
    if( !strcmp(v1,"status") ) {
      if( strstr(pagelevels,v3) ) {
        memcpy(data,"page  ",6); /* Change msg type */
        debug("Sending: ** %s ** to %s\n",data,bbpage);
        bbsend(bbpage,data);      /* SEND IT ACROSS TO BBPAGER */
      }
    }
  }

}

bbsend(ip, data)
char *ip;				/* IP ADDRESS */
char *data;				/* DATA TO SEND ACROSS */
{
 	int n, sockfd, port, timeout();
        struct sockaddr_in serv_addr;

	port=PORT;			/* DEFAULT PORT */

	ipaddr = ip;			/* GLOBAL IP ADDRESS... */

        bzero((char *) &serv_addr, sizeof(serv_addr));
        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = inet_addr(ip);
        serv_addr.sin_port = htons(port);

	/*
	 * THIS IS A LITTLE NASTY
	 * IF THE SERVER'S NOT UP, THIS WILL WAIT A LONG TIME BEFORE
	 * IT TIMES OUT.  SO HANDLE THAT HERE... IT GETS 3 SECONDS TO ANSWER
	 */
 	if (setjmp(env) != 0) {
                debug("CALLING SETJMP\n");
        }
        signal(SIGALRM, (void *)timeout);
        alarm(3);

        /* OPEN SOCKET */
        if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
                fprintf(stderr, "bb: Can't open stream socket\n");
		exit(1);
        }
        if (connect(sockfd, (struct sockaddr *) &serv_addr,
                        sizeof(serv_addr)) < 0) {
                fprintf(stderr, "bb: CAN'T CONNECT TO bbd SERVER @ %s\n", ip);
        }
        n = send(sockfd, data, strlen(data) + 1, 0);
	alarm(0);	/* Previous alarm not needed anymore */
	debug("Send returned: %d\n",n);
	close(sockfd);
	return(n);
}

timeout()
{
	fprintf(stderr, "bb: CONNECTION TO %s TIMED OUT...\n",ipaddr);
	exit(2);
}
