/*
 * bbnet.c
 * BIG BROTHER NETWORK TESTING PROGRAM
 * Sean MacGuire - The MacLawran Group Inc.
 * Version 1.09
 * Feb 2nd, 1999
 *
 * This program is Copyright (c) 1997-1999
 * The MacLawran Group Inc.
 * All Rights Reserved
 *
 * Format: bbnet machine:port ARG 
 *
 * i.e.:	http://blah.com/bobo	HTTP TESTING USING HEAD COMMAND
 *		http://blah.com/http://bobo.com		PROXY TESTING
 * 		http://blah.com "GET /"	GET AN ENTIRE PAGE AS TEST
 *		blah.com		TELNET TESTING (OR ANY OTHER PORT)
 *
 * bbnet understands real machine names and port numbers
 * i.e. blah.com:21 will open a connection to the FTP server
 *
 * Displays whatever answers on that port/address to stdout
 *
 * Returns: 	0	OK
 *		1	Host not found
 *		2	Can't connect to server
 *		3	Timeout
 */

#include "bb.h"		/* THE BIG BROTHER INCLUDE FILE */
#include <netdb.h>	/* FOR HOST INFO */

int http=0;		/* ARE WE HTTP TESTING? */
unsigned long ip;	/* THE IP ADDRESS WE'RE TESTING */
int port;		/* THE PORT NUMBER WE'RE TESTING */
char *dir;		/* DIRECTORY */
char *machine;		/* MACHINE NAME ONLY */

jmp_buf env;  

main(argc, argv)
int argc;
char *argv[];
{
	if (argc < 2 || argc > 3) {	/* ONLY 2 OR 3 ARGUMENTS ARE VALID */
		fprintf(stderr, "bbnet: incorrect number of arguments\n");
		fprintf(stderr, "Format: bbnet machine[:port]|http-request\n");
		exit(1);
	}
	/*
	 * PARSE THE ARGUMENT.  THIS SHOULD BE A DROP-IN REPLACEMENT
	 * FOR lynx, UGH.  THAT MEANS THE FIRST ARGUMENT MAY BE A REALLY
	 * NASTY THING.
	 */
	machine = (char *)malloc(MAXLINE);
	strcpy(machine, argv[1]);

	if (strncmp(machine, "http://", 7) == 0) {
		http = 1;
		debug("We have an http request\n");
		machine += 7;
		port = 80;	/* STANDARD HTTP PORT */
	}
	else port = 23;		/* TELNET DEFAULT */
	setup();		/* DO SETUP */
	if (argc==3) bnetsend(argv[2]);	/* DO TEST */
	else bnetsend((char *)NULL);
	exit(0);
}

/*
 * THIS FUNCTION SETS UP EVERYTHING WE NEED TO CALL THE PORT
 */
setup()
{
	char *eob = 0, *ptr,*newdir;
	struct hostent *ho = 0;

	ptr = (char *)strchr(machine, ':');
	if (ptr != (char *)NULL) {
		debug("SCANNING FOR A PORT %s\n", ptr);
		/*
		 * SMM v1.06b
		 * FOR PROXY TESTING... THERE WILL BE ANOTHER
		 * COLON ':' BUT IT'LL BE FOR THE http: REQUEST
		 * SO CHECK THIS BY MAKING SURE THERE IS NO PORT #
		 */
		if (sscanf(ptr, ":%d", &port) != 0) eob = ptr;
	}
	debug("GOT PORT: %d\n", port);
	dir = (char *)strchr(machine, '/');

	if (dir) {
		/*
		 * CHECK FOR A PROXY REQUEST
		 */
		if (strncmp(dir, "/http://", 8) == 0) {
			if (eob == 0) eob = dir;	
			dir++;		/* SKIP LEADING SLASH */
			debug("PROXY REQUEST for %s\n", dir);
		}
		newdir = (char *)malloc(strlen(dir) + 1);
		strcpy(newdir, dir);
		debug("DIRECTORY = %s\n", newdir);
		*dir = '\0';
	}
	/*
         * From: Doug White <dwhite@gdi.uoregon.edu>
	 */
	else  /* no dir was specified, default to / */
	{
		debug("NO DIR\n");
		newdir = (char *)malloc(2);
 		strcpy(newdir, "/");
	}

	if (eob) *eob = '\0';		/* NOW GET MACHINE */

	debug("BOX: %s\n", machine);

	if((ip = inet_addr(machine)) == -1) { /* IS IT IN DOT FMT */
		if (*machine == '\0') {
			debug("BOX IS LOCAL\n");
			ho = gethostbyname("localhost");
		}
		else ho = gethostbyname(machine);

		if (ho != (struct hostent *)NULL) {
			ip = (unsigned long)ho->h_addr;
		}
		else {	/* TEST FOR AN IP-ADDRESS */
			fprintf(stderr, "bbnet: Unknown host: %s\n", machine);
			exit(1);	/* NO HOST */
		}
	/*
	 * IF WE MAKE IT HERE, WE HAVE TO COPY THE HOSTNAME 
	 * INTO THE ADDRESS STRUCTURE
	 */
		memcpy(&ip, ho->h_addr, ho->h_length);
	}
	dir=newdir;			/* SET DIRECTORY */
}

bnetsend(input)
char *input;
{
 	int n, sockfd, noinput(), timeout();
        struct sockaddr_in serv_addr;
	char line[MAXLINE];

        bzero((char *) &serv_addr, sizeof(serv_addr));
        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = ip;
        serv_addr.sin_port = htons(port);

	/*
	 * THIS IS A LITTLE NASTY
	 * IF THE SERVER'S NOT UP, THIS WILL WAIT A LONG TIME BEFORE
	 * IT TIMES OUT.  SO HANDLE THAT HERE... IT GETS 30 SECONDS TO ANSWER
	 */
 	if (setjmp(env) != 0) {
                debug("CALLING SETJMP\n");
        }
        signal(SIGALRM, (void *)timeout);
        alarm(30);

        /* OPEN SOCKET */
        if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
                fprintf(stderr, "bbnet: CAN'T OPEN SOCKET\n");
		exit(2);
        }
        if (connect(sockfd, (struct sockaddr *) &serv_addr,
                        sizeof(serv_addr)) < 0) {
                fprintf(stderr, "bbnet: CAN'T CONNECT TO SERVER ");
                fprintf(stderr, "@ %s ON PORT %d\n", machine, port);
		/*
		 * THANKS TO: Douwe Dijkstra <D.Dijkstra@Twinfo.nl>
		 */ 
		exit(2);
        }
	/*
	 * HTTP REQUESTS ARE SPECIAL
	 * PROPER HEAD PROTOCOL NOW INCLUDED...
	 * Thanks: Don Sullivan (dsullivan@gaia.arc.nasa.gov)
	 * EXTRA \r FOR Brian Lenihan <brianl@real.com> AND ROXEN
	 * VIRTUAL WEB SERVER SUPPORT FROM: Gabriel Wachob <gwachob@findlaw.com>
	 */
	if (http) {
		if (dir) sprintf(line, "HEAD %s HTTP/1.0\r\nHost: %s\r\n\r\n", dir,machine);
		else sprintf(line, "HEAD / HTTP/1.0\r\nHost: %s\r\n\r\n", machine);
	}
	else {
		sprintf(line, "quit\r\n");
		debug("We have default: %s\n", line);
	}
	/*
 	 * Thanks to Jon Lewis <jlewis@inorganic5.fdt.net>
	 */
	if (input) {
		if (strlen(input)) {
			strncpy(line, input, MAXLINE -5);
			strcat(line, "\r\n");
			debug("We have input: %s\n", line);
		}
		else {	/* THIS IS TO ALLOW NULL INPUT */
			line[0] = '\0';
			debug("NULL INPUT\n");
		}
	}

        n = send(sockfd, line, strlen(line),0);

	line[0]='\0';

        signal(SIGALRM, (void *)noinput);
        alarm(3);
	while (n = recv(sockfd, line, MAXLINE, 0)) {
        	alarm(3);
		if (n > 0) line[n]='\0';/* Shun-Jee Liu <liu@hermes.com.tw> */
		/*
 		 * SMM - 26 DEC 97
 		 * SOME TESTING SHOULDN'T BE DISPLAYED 'CAUSE IT'S GROSS
 		 * THANKS TO: Per.E.Berger@telia.se
 		 */
		if (port == 23) {
			puts("Telnet test OK");
			return(0);
		}
		else puts(line);		/* DUMP OUT THE LINE */
	}
	close(sockfd);
	return(n);
}

timeout()
{
	fprintf(stderr, "bbnet: TIMEOUT %s PORT %d...\n", machine, port);
	exit(3);
}

/*
 * NO INPUT - DON'T WAIT FOREVER!
 */
noinput()
{
	exit(0);
}

